from typing import cast

from inject import autoparams

from ytrssil.bindings import setup_dependencies
from ytrssil.datatypes import Video
from ytrssil.protocols import Client


def get_new_videos() -> list[Video]:
    setup_dependencies()

    @autoparams()
    def _get_new_videos(client: Client) -> list[Video]:
        return client.get_new_videos()

    return cast(list[Video], _get_new_videos())


def get_new_video_count() -> int:
    return len(get_new_videos())



from collections.abc import Iterator
from os import execv, fork
from subprocess import PIPE, Popen
from sys import argv, stderr
from typing import Any

from inject import autoparams

from ytrssil.bindings import setup_dependencies
from ytrssil.config import Configuration
from ytrssil.datatypes import Video
from ytrssil.protocols import Client


def user_query(videos: list[Video], reverse: bool = False) -> list[str]:
    p = Popen(
        ["fzf", "-m"],
        stdout=PIPE,
        stdin=PIPE,
    )
    video_list: Iterator[Video]
    if reverse:
        video_list = reversed(videos)
    else:
        video_list = iter(videos)

    input_bytes = "\n".join(map(str, video_list)).encode("UTF-8")
    stdout, _ = p.communicate(input=input_bytes)
    videos_str: list[str] = stdout.decode("UTF-8").strip().split("\n")
    ret: list[str] = []
    for video_str in videos_str:
        if video_str == "":
            continue

        *_, video_id = video_str.split(" - ")

        try:
            ret.append(video_id)
        except KeyError:
            pass

    return ret


@autoparams()
def fetch(client: Client) -> int:
    client.fetch()
    return 0


@autoparams()
def register(client: Client) -> int:
    client.register()
    return 0


@autoparams("client")
def subscribe_to_channel(client: Client, channel_id: str) -> int:
    client.subscribe_to_channel(channel_id)
    return 0


@autoparams()
def watch_videos(config: Configuration, client: Client) -> int:
    videos = client.get_new_videos()
    if not videos:
        print("No new videos", file=stderr)
        return 1

    selected_videos = user_query(videos)
    if not selected_videos:
        print("No video selected", file=stderr)
        return 2

    video_urls = [
        f"https://www.youtube.com/watch?v={video_id}" for video_id in selected_videos
    ]
    cmd = ["/usr/bin/mpv", *config.mpv_options, *video_urls]
    if fork() == 0:
        execv(cmd[0], cmd)

    for video_id in selected_videos:
        client.mark_video_as_watched(video_id)

    return 0


@autoparams()
def print_url(client: Client) -> int:
    videos = client.get_new_videos()
    if not videos:
        print("No new videos", file=stderr)
        return 1

    selected_videos = user_query(videos)
    if not selected_videos:
        print("No video selected", file=stderr)
        return 2

    for video_id in selected_videos:
        client.mark_video_as_watched(video_id)
        print(f"https://www.youtube.com/watch?v={video_id}")

    return 0


@autoparams()
def mark_as_watched(client: Client) -> int:
    videos = client.get_new_videos()
    if not videos:
        print("No new videos", file=stderr)
        return 1

    selected_videos = user_query(videos)
    if not selected_videos:
        print("No video selected", file=stderr)
        return 2

    for video_id in selected_videos:
        client.mark_video_as_watched(video_id)

    return 0


@autoparams()
def watch_history(config: Configuration, client: Client) -> int:
    videos = client.get_watched_videos()
    if not videos:
        print("No new videos", file=stderr)
        return 1

    selected_videos = user_query(videos)
    if not selected_videos:
        print("No video selected", file=stderr)
        return 2

    video_urls = [
        f"https://www.youtube.com/watch?v={video_id}" for video_id in selected_videos
    ]
    cmd = ["/usr/bin/mpv", *config.mpv_options, *video_urls]
    if fork() == 0:
        execv(cmd[0], cmd)

    return 0


@autoparams()
def mark_as_unwatched(client: Client) -> int:
    videos = client.get_watched_videos()
    if not videos:
        print("No new videos", file=stderr)
        return 1

    selected_videos = user_query(videos)
    if not selected_videos:
        print("No video selected", file=stderr)
        return 2

    for video_id in selected_videos:
        client.mark_video_as_unwatched(video_id)

    return 0


def main(args: list[str] = argv) -> Any:
    setup_dependencies()
    command: str
    try:
        command = args[1]
    except IndexError:
        command = "watch"

    if command == "fetch":
        return fetch()
    elif command == "register":
        return register()
    elif command == "subscribe":
        if len(args) < 3:
            print(
                "Missing channel ID argument for subscribe command",
                file=stderr,
            )
            return 1

        return subscribe_to_channel(channel_id=args[2])
    elif command == "watch":
        return watch_videos()
    elif command == "print":
        return print_url()
    elif command == "history":
        return watch_history()
    elif command == "mark":
        return mark_as_watched()
    elif command == "unmark":
        return mark_as_unwatched()
    else:
        print(f'Unknown command "{command}"', file=stderr)
        print(
            "Available commands: fetch, watch, print, history, mark, unmark",
            file=stderr,
        )
        return 1

    return 0



from collections.abc import Sequence

from ytrssil.api import get_new_video_count, get_new_videos
from ytrssil.datatypes import Video

__all__: Sequence[str] = (
    'Video',
    'get_new_video_count',
    'get_new_videos',
)



from dataclasses import dataclass
from datetime import datetime
from typing import Optional


@dataclass
class Video:
    video_id: str
    title: str
    channel_name: str
    published_timestamp: datetime
    short: bool
    watch_timestamp: Optional[datetime] = None

    def __str__(self) -> str:
        return f"{self.channel_name} - {self.title} - {self.video_id}"


@dataclass
class Channel:
    channel_id: str
    name: str

    def __str__(self) -> str:
        return f"{self.name} - {self.channel_id}"



import requests
from inject import autoparams

from ytrssil.config import Configuration
from ytrssil.datatypes import Video


class HttpClient:
    @autoparams("config")
    def __init__(self, config: Configuration) -> None:
        self.base_url: str = config.api_url
        self.auth: dict[str, str] = {"Authorization": config.token}

    def fetch(self) -> None:
        resp = requests.post(url=f"{self.base_url}/fetch")
        resp.raise_for_status()

    def subscribe_to_channel(self, channel_id: str) -> None:
        resp = requests.post(
            url=f"{self.base_url}/api/channels/{channel_id}/subscribe",
            headers=self.auth,
        )
        resp.raise_for_status()

    def get_new_videos(self) -> list[Video]:
        resp = requests.get(
            url=f"{self.base_url}/api/videos/new",
            headers=self.auth,
        )
        resp.raise_for_status()
        data = resp.json()
        ret: list[Video] = []
        for video_data in data["videos"]:
            ret.append(Video(**video_data))

        return ret

    def get_watched_videos(self) -> list[Video]:
        resp = requests.get(
            url=f"{self.base_url}/api/videos/watched",
            headers=self.auth,
        )
        resp.raise_for_status()
        data = resp.json()
        ret: list[Video] = []
        for video_data in data["videos"]:
            ret.append(Video(**video_data))

        return ret

    def mark_video_as_watched(self, video_id: str) -> None:
        resp = requests.post(
            url=f"{self.base_url}/api/videos/{video_id}/watch",
            headers=self.auth,
        )
        resp.raise_for_status()

    def mark_video_as_unwatched(self, video_id: str) -> None:
        resp = requests.post(
            url=f"{self.base_url}/api/videos/{video_id}/unwatch",
            headers=self.auth,
        )
        resp.raise_for_status()



import json
import os
from dataclasses import dataclass
from typing import Literal


@dataclass
class Configuration:
    token: str
    api_url: str = "https://ytrssil.theedgeofrage.com"
    max_resolution: Literal["480", "720", "1080", "1440", "2160"] = "1440"

    @property
    def mpv_options(self) -> list[str]:
        return [
            "--no-terminal",
            (f"--ytdl-format=bestvideo[height<=?{self.max_resolution}]+bestaudio/best"),
        ]


def load_config() -> Configuration:
    config_prefix: str
    try:
        config_prefix = os.environ["XDG_CONFIG_HOME"]
    except KeyError:
        config_prefix = os.path.expanduser("~/.config")

    config_path: str = os.path.join(config_prefix, "ytrssil", "config.json")
    with open(config_path) as f:
        config_data = json.load(f)

    return Configuration(**config_data)



from inject import Binder, Injector, clear_and_configure, get_injector_or_die

from ytrssil.client import HttpClient
from ytrssil.config import Configuration, load_config
from ytrssil.protocols import Client


def dependency_configuration(binder: Binder) -> None:
    binder.bind(Configuration, load_config())
    binder.bind_to_constructor(Client, HttpClient)


def setup_dependencies() -> Injector:
    clear_and_configure(dependency_configuration)

    return get_injector_or_die()
